import os
from typing import Dict

from srai_core.store.bytes_store_base import BytesStoreBase
from srai_core.tools.tools_s3 import create_client_and_resource_s3


class BytesStoreS3(BytesStoreBase):
    # file store for
    def __init__(
        self, aws_access_key_id: str, aws_secret_access_key: str, aws_name_region: str, name_bucket: str
    ) -> None:
        self.aws_name_region = aws_name_region
        self.name_bucket = name_bucket

        client_s3, resource_s3 = create_client_and_resource_s3(
            aws_access_key_id, aws_secret_access_key, aws_name_region
        )
        self.client_s3 = client_s3
        self.resource_s3 = resource_s3

    def exists_bytes(self, bytes_id: str) -> bool:
        try:
            self.client_s3.head_object(Bucket=self.name_bucket, Key=bytes_id)
            return True
        except Exception as e:
            return False

    def count_bytes(self) -> int:
        return len(self.client_s3.list_objects(Bucket=self.name_bucket)["Contents"])

    def load_bytes(self, bytes_id: str) -> bytes:
        return self.resource_s3.Object(self.name_bucket, bytes_id).get()["Body"].read()

    def load_bytes_all(self) -> Dict[str, bytes]:
        dict_bytes = {}
        list_object = self.client_s3.list_objects(Bucket=self.name_bucket)
        if "Contents" not in list_object:
            return dict_bytes
        for object in list_object["Contents"]:
            bytes_id = object["Key"]
            dict_bytes[bytes_id] = self.load_bytes(bytes_id)
        return dict_bytes

    def delete_bytes(self, bytes_id: str) -> None:
        self.client_s3.delete_object(Bucket=self.name_bucket, Key=bytes_id)

    def delete_bytes_all(self) -> int:
        count = self.count_bytes()
        list_object = self.client_s3.list_objects(Bucket=self.name_bucket)
        if "Contents" not in list_object:
            return 0
        for object in list_object["Contents"]:
            self.delete_bytes(object["Key"])
        return count

    def save_bytes(self, bytes_id: str, bytes: bytes) -> None:
        self.resource_s3.Object(self.name_bucket, bytes_id).put(Body=bytes)

    def get_bytes_url(self, bytes_id: str) -> str:
        return f"https://s3-{self.aws_name_region}.amazonaws.com/{self.name_bucket}/{bytes_id}"


# class BytessourceS3(Bytessource):

#     def __init__(self, config: dict, aws_name_region, name_bucket, prefix_root, list_key=[]) -> None:
#         super().__init__(config)
#         self.validate_list_key(list_key)
#         self.aws_name_region = aws_name_region

#         self.name_bucket = name_bucket
#         self.prefix_root = prefix_root
#         self.list_key = list_key
#         self.name_object = prefix_root + "/".join(list_key)

#         self.aws_access_key_id = self.config["aws_access_key_id"]
#         self.aws_secret_access_key = self.config["aws_secret_access_key"]
#         client_s3, resource_s3 = ToolsS3.create_client_and_resource_s3(
#             self.aws_name_region, self.aws_access_key_id, self.aws_secret_access_key
#         )
#         self.client_s3 = client_s3
#         self.resource_s3 = resource_s3

#     def _prefix(self):
#         prefix = self.prefix_root
#         if 0 < len(self.list_key):
#             prefix += "/".join(self.list_key) + "/"
#         return prefix

#     def url_public(self) -> str:
#         location = self.client_s3.get_bucket_location(Bucket=self.name_bucket)["LocationConstraint"]
#         return "https://s3-%s.amazonaws.com/%s/%s" % (location, self.name_bucket, self.name_object)

#     def is_public_save(self, is_public) -> None:
#         ToolsS3.object_is_public_save(self.client_s3, self.resource_s3, self.name_bucket, self.name_object, is_public)

#     def is_public_load(self) -> bool:
#         return ToolsS3.object_is_public_load(self.client_s3, self.resource_s3, self.name_bucket, self.name_object)
#         # self._is_public = is_public
#         # ToolsS3.object_set_public(self.client_s3, self.resource_s3, self.name_bucket, self.name_object, is_public)

#     def exists(self) -> bool:
#         return ToolsS3.object_exists(self.client_s3, self.resource_s3, self.name_bucket, self.name_object)

#     def list_shallow(self) -> None:
#         prefix = self._prefix()
#         list_name_object = ToolsS3.list_name_object_for_prefix(
#             self.client_s3, self.resource_s3, self.name_bucket, prefix
#         )
#         list_list_key = []
#         for name_object in list_name_object:
#             postfix = name_object[len(prefix) :]
#             if not "/" in name_object[len(prefix) :]:
#                 list_list_key.append(postfix.split("/"))
#         return list_list_key

#     def list_deep(self) -> None:
#         prefix = self._prefix()
#         list_name_object = ToolsS3.list_name_object_for_prefix(
#             self.client_s3, self.resource_s3, self.name_bucket, prefix
#         )
#         list_list_key = []
#         for name_object in list_name_object:
#             postfix = name_object[len(prefix) :]
#             list_list_key.append(postfix.split("/"))
#         return list_list_key

#     def write(self, bytearray_object: bytearray) -> None:
#         ToolsS3.object_save(self.client_s3, self.resource_s3, self.name_bucket, self.name_object, bytearray_object)

#     def read(self) -> bytearray:
#         return ToolsS3.object_load(self.client_s3, self.resource_s3, self.name_bucket, self.name_object)

#     def delete(self) -> None:
#         ToolsS3.object_delete(self.client_s3, self.resource_s3, self.name_bucket, self.name_object)

#     def size(self):
#         return ToolsS3.object_size(self.client_s3, self.resource_s3, self.name_bucket, self.name_object)

#     def join(self, list_key: List[str]):
#         self.validate_list_key(list_key)
#         list_key_extended = self.list_key.copy()
#         list_key_extended.extend(list_key)
#         return BytessourceS3(self.config, self.aws_name_region, self.name_bucket, self.prefix_root, list_key_extended)

#     def list_for_prefix(self, list_key_prefix: List[str]) -> "List[List[str]]":
#         self.validate_list_key(list_key_prefix)
#         prefix = "/".join(list_key_prefix)
#         list_name_object = ToolsS3.list_name_object_for_prefix(
#             self.client_s3, self.resource_s3, self.name_bucket, prefix
#         )
#         list_list_key = []
#         for name_object in list_name_object:
#             list_list_key.append(name_object.split("/"))
#         return list_list_key

#     def to_dict(self) -> "dict":
#         dict_bytessource = {}
#         dict_bytessource["type_bytessource"] = "BytessourceS3"
#         dict_bytessource["aws_name_region"] = self.aws_name_region
#         dict_bytessource["name_bucket"] = self.name_bucket
#         dict_bytessource["prefix_root"] = self.prefix_root
#         dict_bytessource["list_key"] = self.list_key.copy()
#         return dict_bytessource

#     @staticmethod
#     def from_dict(config: dict, dict_bytessource) -> "BytessourceS3":
#         if not dict_bytessource["type_bytessource"] == "BytessourceS3":
#             raise Exception("incorrect_dict_type")

#         return BytessourceS3(
#             config,
#             dict_bytessource["aws_name_region"],
#             dict_bytessource["name_bucket"],
#             dict_bytessource["prefix_root"],
#             dict_bytessource["list_key"],
#         )



import subprocess


def main():
    print("Delete old distribution")
    subprocess.run("rm -rf dist", shell=True, check=True)

    print("Creating distribution")
    subprocess.run("python setup.py sdist", shell=True, check=True)

    command = "twine upload dist/*"
    print(f"Running command: {command}")
    subprocess.run(command, shell=True, check=True)


if __name__ == "__main__":
    main()



import asyncio
import json
import sys

from srai_core import command_handler_base
from srai_core.command_handler_base import CommandHandlerBase
from srai_core.tools_docker import build_docker_async, release_docker_local_to_aws_async


async def srai_release():
    if len(sys.argv) < 2:
        print("Usage: deploy_docker deployment_file.json")
        sys.exit(1)

    path_deployment_file = sys.argv[1]
    with open(path_deployment_file) as f:
        deployment = json.load(f)
    for build_target in deployment["list_build_target"]:
        command_handler = CommandHandlerBase.from_dict(build_target)
        await build_docker_async(command_handler)

    for release_target in deployment["list_release_target"]:
        command_handler = CommandHandlerBase.from_dict(release_target)
        release_type = release_target["release_type"]
        if release_type == "release_docker_local_to_aws":
            await release_docker_local_to_aws_async(command_handler)
        else:
            print(f"Unknown release_type: {release_type}")
            sys.exit(1)


def main():
    asyncio.run(srai_release())


if __name__ == "__main__":
    main()



from srai_core.store.bytes_store_base import BytesStoreBase
from srai_core.store.bytes_store_mongo import BytesStoreMongo
from srai_core.store.database_base import DatabaseBase
from srai_core.store.document_store_base import DocumentStoreBase
from srai_core.store.document_store_mongo import DocumentStoreMongo


class DatabaseMongo(DatabaseBase):
    def __init__(self, database_name: str, connection_string: str):
        super().__init__(database_name)
        self.connection_string = connection_string

    def get_document_store(self, collection_name: str) -> DocumentStoreBase:
        return DocumentStoreMongo(self.connection_string, self.database_name, collection_name)

    def get_bytes_store(self, collection_name: str) -> BytesStoreBase:
        return BytesStoreMongo(self.connection_string, self.database_name, collection_name)



import json
import os
import sys
import time


def load_list_image_description(
    client_ec2,
    resource_ec2,
):
    # self | amazon | aws-marketplace

    if os.path.isfile("list_image_description.json"):
        with open("list_image_description.json", "r") as file:
            return json.load(file)

    list_image_description = client_ec2.describe_images(Owners=["amazon"])["Images"]
    with open("list_image_description.json", "w") as file:
        json.dump(list_image_description, file)
    return list_image_description


def load_list_name_region(
    client_ec2,
    resource_ec2,
):
    response = client_ec2.describe_regions()
    return [region["RegionName"] for region in response["Regions"]]
    # print('Regions:', response['Regions'])


def load_list_name_availability_zone(
    client_ec2,
    resource_ec2,
):
    response = client_ec2.describe_availability_zones()
    return [region["ZoneName"] for region in response["AvailabilityZones"]]


def load_list_sir_id(client_ec2, resource_ec2):
    response = client_ec2.describe_spot_instance_requests()
    list_sir_id_pending = []
    list_sir_id_fulfilled = []
    # print(response)
    for sir in response["SpotInstanceRequests"]:
        if sir["Status"]["Code"] == "fulfilled":
            list_sir_id_fulfilled.append(sir["SpotInstanceRequestId"])
        else:
            list_sir_id_pending.append(sir["SpotInstanceRequestId"])
    return list_sir_id_pending, list_sir_id_fulfilled


def load_list_instance_id_from_list_sir_id(client_ec2, resource_ec2, list_sir_id):
    list_filter = [{"Name": "spot-instance-request-id", "Values": list_sir_id}]
    response = client_ec2.describe_spot_instance_requests(Filters=list_filter)
    list_instance_id = []
    for sir in response["SpotInstanceRequests"]:
        if sir["Status"]["Code"] == "fulfilled":
            list_instance_id.append(sir["InstanceId"])
    return list_instance_id


def load_instance_id_from_sir_id(client_ec2, resource_ec2, sir_id):
    list_instance_id = load_list_instance_id_from_list_sir_id(client_ec2, resource_ec2, [sir_id])
    if len(list_instance_id) == 1:
        return list_instance_id[0]
    else:
        print()
        return None


def load_list_instance_description(client_ec2, resource_ec2, dict_tag={}):
    response = client_ec2.describe_instances()
    list_instance_description = []
    for list_instance_description_part in response["Reservations"]:
        for instance_description in list_instance_description_part["Instances"]:
            list_instance_description.append(instance_description)
    return list_instance_description


def load_list_instance_id(client_ec2, resource_ec2):
    list_instance_description = load_list_instance_description(
        client_ec2,
        resource_ec2,
    )
    return [instance_description["InstanceId"] for instance_description in list_instance_description]


def instance_launch_spot(
    client_ec2,
    resource_ec2,
    type_instance: str,
    name_availability_zone: str,
):
    # , count=1, is_spot=False, name_type='generel'
    # type_instance = 't2.medium' # for hosting web services
    # type_instance = 't3.xlarge'
    # size_volume_gb=100,

    price_spot = instance_load_spot_price(
        client_ec2,
        resource_ec2,
        type_instance,
        name_availability_zone,
    )
    print("price: " + price_spot)
    price_spot_bid = str(float(price_spot) * 1.1)
    print("bid: " + price_spot_bid)
    sys.stdout.flush()

    sir_id = instance_request_spot(client_ec2, resource_ec2, price_spot_bid, type_instance)
    print(sir_id)
    sys.stdout.flush()
    sir_await_fulfilled(client_ec2, resource_ec2, sir_id)
    instance_id = load_instance_id_from_sir_id(client_ec2, resource_ec2, sir_id)
    print(instance_id)
    sys.stdout.flush()
    instance_await_running(client_ec2, resource_ec2, instance_id)
    return instance_id

    # image_id = 'ami-04b29aaed8d74f8f3' #Machine learning instance AMI (at least 75 GB)
    # type_instance = 't2.medium'
    # return self.instance_launch(type_instance=type_instance, image_id=image_id, size_volume_gb=100, address_id=None)


# in usd per hour
# 'p2.large' has not spot price? should be like $0.3094 per Hour
# def instance_load_spot_price(client_ec2, resource_ec2, type_instance='t2.medium'):
def instance_load_spot_price(
    client_ec2,
    resource_ec2,
    type_instance: str,
    name_availability_zone: str,
):
    list_type_instance = [type_instance]
    response = client_ec2.describe_spot_price_history(
        InstanceTypes=list_type_instance,
        MaxResults=1,
        ProductDescriptions=["Linux/UNIX (Amazon VPC)"],
        AvailabilityZone=name_availability_zone,
    )

    # print(response['SpotPriceHistory'])
    return response["SpotPriceHistory"][0]["SpotPrice"]


def instance_request_spot(client_ec2, resource_ec2, spot_price, type_instance="t2.medium"):
    launch_specification = create_launch_specification_spot(client_ec2, resource_ec2, type_instance)
    response = client_ec2.request_spot_instances(
        InstanceCount=1,
        LaunchSpecification=launch_specification,
        SpotPrice=spot_price,
        Type="one-time",
    )
    # print(response)
    sir_id = response["SpotInstanceRequests"][0]["SpotInstanceRequestId"]
    return sir_id


def sir_await_fulfilled(client_ec2, resource_ec2, sir_id):
    print("await spot_instance_request_fulfilled")
    sys.stdout.flush()
    waiter = client_ec2.get_waiter("spot_instance_request_fulfilled")
    waiter.wait(SpotInstanceRequestIds=[sir_id])
    instance_id = load_instance_id_from_sir_id(client_ec2, resource_ec2, sir_id)
    while not instance_id:
        time.sleep(1)
        instance_id = load_instance_id_from_sir_id(client_ec2, resource_ec2, sir_id)
    print("spot_instance_request_fulfilled")
    sys.stdout.flush()
    return instance_id


def load_list_sir_description(client_ec2):
    response = client_ec2.describe_spot_instance_requests()
    return response["SpotInstanceRequests"]


# TODO use this for regular ones as well
def create_launch_specification_spot(
    client_ec2,
    resource_ec2,
    type_instance="t2.medium",
    name_availability_zone="us-east-1a",
):
    name_key = "south_river_main"
    id_image = "ami-04b29aaed8d74f8f3"  # Machine learning instance AMI (at least 75 GB) #TODO replace this a we are using mostly docker

    # list_security_group = [{'GroupName': 'string','GroupId': 'string'}]
    list_security_group_id = ["sg-001a357692a7d0ee7"]  # Any
    list_mapping_block_device = [
        {
            "DeviceName": "/dev/sda1",
            "Ebs": {
                "DeleteOnTermination": True,
                "SnapshotId": "snap-0f968e137b5347031",
                "VolumeSize": 100,
                "VolumeType": "gp2",
                "Encrypted": False,
            },
        }
    ]
    launch_specification = {}
    launch_specification["ImageId"] = id_image
    launch_specification["KeyName"] = name_key
    # launch_specification['SecurityGroups'] = list_security_group
    launch_specification["SecurityGroupIds"] = list_security_group_id
    launch_specification["InstanceType"] = type_instance
    launch_specification["Placement"] = {"AvailabilityZone": name_availability_zone}
    launch_specification["BlockDeviceMappings"] = list_mapping_block_device
    return launch_specification


# def instance_launch_small(client_ec2,resource_ec2, ):
#     image_id = '' #Machine learning instance AMI (at least 75 GB)
#     type_instance = '' #get the smallest amount of
#     # spin up a machine
#     return self.instance_launch(type_instance=type_instance, image_id=image_id, size_volume_gb=100, address_id=None)


def instance_launch_general(client_ec2, resource_ec2, size_volume_gb=100, address_id=None):
    image_id = "ami-04b29aaed8d74f8f3"  # Machine learning instance AMI (at least 75 GB)
    type_instance = "t2.medium"
    return instance_launch(
        client_ec2,
        resource_ec2,
        type_instance=type_instance,
        image_id=image_id,
        size_volume_gb=100,
        address_id=None,
    )


# def instance_launch_train(client_ec2,resource_ec2, ):
#     image_id = 'ami-04b29aaed8d74f8f3' #Machine learning instance AMI (at least 75 GB)
#     type_instance = 'p2.large' #get the smallest amount of
#     # spin up a machine
#     return self.instance_launch(type_instance=type_instance, image_id=image_id, size_volume_gb=100, address_id=None)


def instance_launch(client_ec2, resource_ec2, type_instance: str, size_volume_gb=100, address_id=None):
    # spin up a machine
    KeyName = "south_river_main"
    InstanceType = type_instance
    ImageId = "ami-04b29aaed8d74f8f3"
    SecurityGroupIds = ["sg-001a357692a7d0ee7"]  # Any
    BlockDeviceMappings = [
        {
            "DeviceName": "/dev/sda1",
            "Ebs": {
                "DeleteOnTermination": True,
                "SnapshotId": "snap-0f968e137b5347031",
                "VolumeSize": 100,
                "VolumeType": "gp2",
                "Encrypted": False,
            },
        }
    ]

    instance = resource_ec2.create_instances(
        BlockDeviceMappings=BlockDeviceMappings,
        ImageId=ImageId,
        SecurityGroupIds=SecurityGroupIds,
        InstanceType=InstanceType,
        KeyName=KeyName,
        MinCount=1,
        MaxCount=1,
    )[0]

    if address_id:  # attach ip
        address = resource_ec2.VpcAddress(address_id)
        address.associate(instance.instance_id)
    return instance.instance_id


def instance_get_hostname(client_ec2, resource_ec2, instance_id):
    hostname = resource_ec2.Instance(instance_id).public_dns_name
    return hostname


def instance_await_running(client_ec2, resource_ec2, instance_id):
    waiter = client_ec2.get_waiter("instance_running")
    waiter.wait(InstanceIds=[instance_id])


def instance_await_terminated(client_ec2, resource_ec2, instance_id):
    waiter = client_ec2.get_waiter("instance_terminated")
    waiter.wait(InstanceIds=[instance_id])


def instance_terminate(client_ec2, resource_ec2, instance_id):
    list_instance_id = [instance_id]
    client_ec2.terminate_instances(InstanceIds=list_instance_id)


def instance_terminate_all(client_ec2, resource_ec2):
    list_instance_id = load_list_instance_id(client_ec2, resource_ec2)
    client_ec2.terminate_instances(InstanceIds=list_instance_id)



import os

from srai_core.command_handler_subprocess import CommandHandlerSubprocess
from srai_core.tools_docker import build_docker, get_image_tag, remove_container, start_container, stop_container


def main():
    command_handler = CommandHandlerSubprocess()
    build_docker(command_handler)
    image_tag = get_image_tag()
    dict_env = {}
    dict_env["SRAI_TELEGRAM_TOKEN"] = os.environ.get("SRAI_TELEGRAM_TOKEN")
    dict_env["AWS_ACCESS_KEY_ID"] = os.environ.get("AWS_ACCESS_KEY_ID")
    dict_env["AWS_SECRET_ACCESS_KEY"] = os.environ.get("AWS_SECRET_ACCESS_KEY")
    dict_env["AWS_REGION_NAME"] = os.environ.get("AWS_REGION_NAME")
    dict_env["JOB_ID"] = os.environ.get("JOB_ID")
    dict_env["IMAGE_TAG"] = image_tag

    container_name = image_tag.split(":")[0].split("/")[-1]

    stop_container(command_handler, container_name)
    remove_container(command_handler, container_name)
    start_container(command_handler, image_tag, container_name, dict_env)


if __name__ == "__main__":
    main()



from abc import ABC, abstractmethod

from srai_core.store.bytes_store_base import BytesStoreBase
from srai_core.store.document_store_base import DocumentStoreBase


class DatabaseBase(ABC):
    def __init__(self, database_name: str):
        self.database_name = database_name

    @abstractmethod
    def get_document_store(self, collection_name: str) -> DocumentStoreBase:
        raise NotImplementedError()

    @abstractmethod
    def get_bytes_store(self, collection_name: str) -> BytesStoreBase:
        raise NotImplementedError()



import json
from typing import Any, Optional, Tuple

import boto3
from botocore.exceptions import ClientError


def create_client_and_resource_s3(
    aws_access_key_id: str,
    aws_secret_access_key: str,
    aws_name_region: str,
) -> Tuple[Any, Any]:
    client_s3 = boto3.client(
        "s3",
        region_name=aws_name_region,
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
    )

    resource_s3 = boto3.resource(
        "s3",
        region_name=aws_name_region,
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
    )

    return client_s3, resource_s3


def bucket_exists(client_s3, resource_s3, bucket_name: str) -> bool:
    # print(type(client_s3))
    # print(type(resource_s3))
    try:
        client_s3.head_bucket(Bucket=bucket_name)
        return True
    except ClientError as e:
        if e.response["Error"]["Code"] == "404":
            return False
        else:
            raise Exception("fail")


def bucket_create_read_only(client_s3, resource_s3, bucket_name: str, aws_region_name: str) -> None:
    # Create the S3 bucket
    try:
        client_s3.create_bucket(Bucket=bucket_name, CreateBucketConfiguration={"LocationConstraint": aws_region_name})
        print(f"Bucket {bucket_name} created successfully.")
    except Exception as e:
        print(f"Error creating bucket: {e}")

    # Bucket policy - Grants read-only access to anonymous users
    bucket_policy = {
        "Version": "2012-10-17",
        "Statement": [
            {
                "Sid": "AddPerm",
                "Effect": "Allow",
                "Principal": "*",
                "Action": "s3:GetObject",
                "Resource": f"arn:aws:s3:::{bucket_name}/*",
            }
        ],
    }

    # Convert the policy to a JSON string
    policy_string = json.dumps(bucket_policy)

    # public_access_block = bucket_public_access_block_load(client_s3, resource_s3, bucket_name)
    # print(f"Public access block configuration: {public_access_block}")
    try:
        # Removing the public access block configuration
        client_s3.delete_public_access_block(Bucket=bucket_name)
        print(f"Public access block removed for bucket: {bucket_name}")
    except Exception as e:
        print(f"Error removing public access block: {e}")

    # Apply the bucket policy
    try:
        client_s3.put_bucket_policy(Bucket=bucket_name, Policy=policy_string)
        print(f"Read-only bucket policy applied to {bucket_name}.")
    except Exception as e:
        print(f"Error applying bucket policy: {e}")
    # Reapplying this this after seems wrong
    # Apply the public access block configuration
    # bucket_public_access_block_save(client_s3, resource_s3, bucket_name, **public_access_block)
    # print(f"Public access block configuration applied to {bucket_name}.")


def bucket_list(client_s3, resource_s3):
    response = client_s3.list_buckets()
    return [bucket["Name"] for bucket in response["Buckets"]]


def bucket_public_access_block_load(client_s3, resource_s3, name_bucket: str) -> dict:
    return client_s3.get_public_access_block(Bucket=name_bucket)["PublicAccessBlockConfiguration"]


def bucket_public_access_block_save(
    client_s3,
    resource_s3,
    name_bucket: str,
    BlockPublicAcls: Optional[bool] = None,
    IgnorePublicAcls: Optional[bool] = None,
    BlockPublicPolicy: Optional[bool] = None,
    RestrictPublicBuckets: Optional[bool] = None,
):
    try:
        dict_pabc = bucket_public_access_block_load(client_s3, resource_s3, name_bucket)
    except ClientError as client_error:
        if client_error.response["Error"]["Code"] == "NoSuchPublicAccessBlockConfiguration":
            dict_pabc = {
                "BlockPublicAcls": False,
                "IgnorePublicAcls": False,
                "BlockPublicPolicy": False,
                "RestrictPublicBuckets": False,
            }
        else:
            raise Exception("fail")
    changed = False
    if (not BlockPublicAcls is None) and (BlockPublicAcls != dict_pabc["BlockPublicAcls"]):
        dict_pabc["BlockPublicAcls"] = BlockPublicAcls
        changed = True

    if (not IgnorePublicAcls is None) and (IgnorePublicAcls != dict_pabc["IgnorePublicAcls"]):
        dict_pabc["IgnorePublicAcls"] = IgnorePublicAcls
        changed = True

    if (not BlockPublicPolicy is None) and (BlockPublicPolicy != dict_pabc["BlockPublicPolicy"]):
        dict_pabc["BlockPublicPolicy"] = BlockPublicPolicy
        changed = True

    if (not RestrictPublicBuckets is None) and (RestrictPublicBuckets != dict_pabc["RestrictPublicBuckets"]):
        dict_pabc["RestrictPublicBuckets"] = RestrictPublicBuckets
        changed = True

    if changed:
        client_s3.put_public_access_block(Bucket=name_bucket, PublicAccessBlockConfiguration=dict_pabc)


def bucket_read_policy(client_s3, resource_s3, name_bucket: str):
    return resource_s3.BucketPolicy(name_bucket).policy


def bucket_delete(client_s3, resource_s3, name_bucket: str):
    resource_s3.Bucket(name_bucket).delete()


def object_exists(client_s3, resource_s3, name_bucket: str, name_object: str):
    try:
        resource_s3.Object(name_bucket, name_object).load()
    except ClientError as client_error:
        if client_error.response["Error"]["Code"] == "404":
            return False
        elif client_error.response["Error"]["Code"] == "403":
            raise Exception("Access denied")
        else:
            raise Exception("fail")
    else:
        return True


@staticmethod
def object_save(client_s3, resource_s3, name_bucket: str, name_object: str, bytearray_object: bytearray):
    object_s3 = resource_s3.Object(name_bucket, name_object)
    object_s3.put(Body=bytearray_object)


@staticmethod
def object_is_public_save(client_s3, resource_s3, name_bucket: str, name_object: str, is_public: bool = False) -> None:
    object_s3_acl = resource_s3.ObjectAcl(name_bucket, name_object)
    if is_public:
        object_s3_acl.put(ACL="public-read")
    else:
        object_s3_acl.put(ACL="private")


@staticmethod
def object_is_public_load(client_s3, resource_s3, name_bucket: str, name_object: str) -> bool:
    object_s3_acl = resource_s3.ObjectAcl(name_bucket, name_object)
    # object_s3_acl.put(ACL='public-read')
    for grant in object_s3_acl.grants:
        if grant["Grantee"]["Type"] == "Group":
            if (
                grant["Permission"] == "READ"
                and grant["Grantee"]["URI"] == "http://acs.amazonaws.com/groups/global/AllUsers"
            ):
                return True
    return False
    # object_s3_acl.put(ACL='private')
    # object_s3_acl.load()
    # print(object_s3_acl.grants)


#    {'Grantee': {'Type': 'Group', 'URI': 'http://acs.amazonaws.com/groups/global/AllUsers'}, 'Permission': 'READ'}
# if is_public:
#     object_s3_acl.put(ACL='public-read')
# else:
#     object_s3_acl.put(ACL='private')


@staticmethod
def object_load(client_s3, resource_s3, name_bucket: str, name_object: str):
    return resource_s3.Object(name_bucket, name_object).get()["Body"].read()


@staticmethod
def object_delete(client_s3, resource_s3, name_bucket: str, name_object: str):
    resource_s3.Object(name_bucket, name_object).delete()


@staticmethod
def object_size(client_s3, resource_s3, name_bucket: str, name_object: str):
    return resource_s3.Bucket(name_bucket).Object(name_object).content_length


@staticmethod
def list_name_object_for_prefix(client_s3, resource_s3, name_bucket, prefix):
    bucket = resource_s3.Bucket(name_bucket)
    object_summary_iterator = bucket.objects.filter(
        Delimiter=",",
        EncodingType="url",
        MaxKeys=1000,
        Prefix=prefix,
    )
    list_name_object = []
    for object_summary in object_summary_iterator:
        list_name_object.append(object_summary.key)

    return list_name_object



import os
from base64 import b64decode
from typing import Any

import boto3


def read_package_info():
    pass


def get_string_from_env(env_name: str) -> str:
    value = os.environ.get(env_name)
    if value is None:
        raise Exception(f"Environment variable {env_name} is not set")
    return value


def get_string_from_env_base64(env_name: str) -> str:
    value_base64 = get_string_from_env(env_name)
    return b64decode(value_base64).decode("utf-8")


def get_path_from_env(env_name: str) -> str:
    path_file = get_string_from_env(env_name)
    if not os.path.exists(path_file):
        raise Exception(f"Path {path_file} does not exist")
    return path_file


def get_client_ec2() -> Any:
    aws_access_key_id = get_string_from_env("AWS_ACCESS_KEY_ID")
    aws_secret_access_key = get_string_from_env("AWS_SECRET_ACCESS_KEY")
    aws_region_name = get_string_from_env("AWS_REGION_NAME")
    return boto3.client(
        "ec2",
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
        region_name=aws_region_name,
    )


def get_resource_ec2() -> Any:
    aws_access_key_id = get_string_from_env("AWS_ACCESS_KEY_ID")
    aws_secret_access_key = get_string_from_env("AWS_SECRET_ACCESS_KEY")
    aws_region_name = get_string_from_env("AWS_REGION_NAME")
    return boto3.resource(
        "ec2",
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
        region_name=aws_region_name,
    )


def get_client_s3() -> Any:
    aws_access_key_id = get_string_from_env("AWS_ACCESS_KEY_ID")
    aws_secret_access_key = get_string_from_env("AWS_SECRET_ACCESS_KEY")
    aws_region_name = get_string_from_env("AWS_REGION_NAME")
    return boto3.client(
        "s3",
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
        region_name=aws_region_name,
    )


def get_client_ecr() -> Any:
    aws_access_key_id = get_string_from_env("AWS_ACCESS_KEY_ID")
    aws_secret_access_key = get_string_from_env("AWS_SECRET_ACCESS_KEY")
    aws_region_name = get_string_from_env("AWS_REGION_NAME")
    return boto3.client(
        "ecr",
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
        region_name=aws_region_name,
    )


def get_client_ecs() -> Any:
    aws_access_key_id = get_string_from_env("AWS_ACCESS_KEY_ID")
    aws_secret_access_key = get_string_from_env("AWS_SECRET_ACCESS_KEY")
    aws_region_name = get_string_from_env("AWS_REGION_NAME")
    return boto3.client(
        "ecs",
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
        region_name=aws_region_name,
    )



from typing import Dict, Optional

from pymongo.mongo_client import MongoClient
from pymongo.server_api import ServerApi

from srai_core.store.document_store_base import DocumentStoreBase


class DocumentStoreMongo(DocumentStoreBase):
    def __init__(self, connection_string: str, database_name: str, collection_name: str):
        self.client = MongoClient(connection_string, server_api=ServerApi("1"))
        self.db = self.client.get_database(database_name)
        self.collection = self.db.get_collection(collection_name)

    def exists_document(self, document_id) -> bool:
        return self.collection.count_documents({"_id": document_id}) > 0

    def count_document(self) -> int:
        return self.collection.count_documents({})

    def load_document(self, document_id: str) -> dict:
        document = self.try_load_document(document_id)
        if document is None:
            raise ValueError(f"Document {document_id} not found")
        return document

    def try_load_document(self, document_id: str) -> Optional[dict]:
        document_result = self.collection.find_one({"_id": document_id})
        if document_result is None:
            return None
        return document_result["document"]

    def load_document_all(self) -> Dict[str, dict]:
        cursor_document_result = self.collection.find({})
        dict_document = {}
        for document_result in cursor_document_result:
            dict_document[document_result["_id"]] = document_result["document"]
        return dict_document

    # TODO: Implement load_document_for_query
    # def load_document_for_query(self, query: Dict[str, str]) -> Dict[str, dict]:
    #     # TODO limit query language
    #     cursor_document_result = self.collection.find(query)
    #     dict_document = {}
    #     for document_result in cursor_document_result:
    #         print(document_result)
    #         dict_document[document_result["_id"]] = document_result["document"]
    #     return dict_document

    def save_document(self, document_id: str, document: dict) -> None:
        self.collection.insert_one({"_id": document_id, "document": document})

    def delete_document(self, document_id: str) -> None:
        query = {"_id": document_id}
        self.collection.delete_one(query)

    def delete_document_all(self) -> int:
        delete_result = self.collection.delete_many({})
        return delete_result.deleted_count



import os

from srai_core.store.bytes_store_base import BytesStoreBase
from srai_core.store.bytes_store_disk import BytesStoreDisk
from srai_core.store.database_base import DatabaseBase
from srai_core.store.document_store_base import DocumentStoreBase
from srai_core.store.document_store_disk import DocumentStoreDisk


class DatabaseDisk(DatabaseBase):
    def __init__(self, database_name: str, path_dir: str):
        super().__init__(database_name)
        self.path_dir = path_dir

    def get_document_store(self, collection_name: str) -> DocumentStoreBase:
        path_dir_store = os.path.join(self.path_dir, collection_name)
        return DocumentStoreDisk(path_dir_store)

    def get_bytes_store(self, collection_name: str) -> BytesStoreBase:
        path_dir_store = os.path.join(self.path_dir, collection_name)
        return BytesStoreDisk(path_dir_store)
